import { createElement } from 'react';
import { ApolloLink, Observable, InMemoryCache, ApolloClient, ApolloProvider } from '@apollo/client';
import { MockLink } from '@apollo/client/testing';
import { buildClientSchema, graphql, print, GraphQLError } from 'graphql';
import { addMockFunctionsToSchema, addResolveFunctionsToSchema } from 'graphql-tools';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

//   url: string,
//   readable: boolean = true,
//   init?: RequestInit
// ): Promise<ITypeDefinitions> {
//   const headers = init ? init.headers : {};
//
//   return fetch(url, {
//     method: 'POST',
//     headers: {
//       Accept: 'application/json',
//       'Content-Type': 'application/json',
//       ...headers,
//     },
//     body: JSON.stringify({
//       query: getIntrospectionQuery,
//     }),
//     ...init,
//   })
//     .then((res) => res.json())
//     .then((schemaJSON) => {
//       if (readable) {
//         return printSchema(buildClientSchema(schemaJSON.data));
//       }
//
//       return JSON.stringify(schemaJSON, null, 2);
//     });
// }

function delay(ms) {
  return new Promise(function (resolve) {
    setTimeout(function () {
      resolve();
    }, ms);
  });
}

function createMockLink(schema, rootValue, context, options) {
  if (rootValue === void 0) {
    rootValue = {};
  }

  if (context === void 0) {
    context = {};
  }

  if (options === void 0) {
    options = {};
  }

  var delayMs = options && options.delay || 200;
  return new ApolloLink(function (operation) {
    return new Observable(function (observer) {
      var query = operation.query,
          operationName = operation.operationName,
          variables = operation.variables;
      delay(delayMs).then(function () {
        return graphql(schema, print(query), rootValue, context, variables, operationName);
      }).then(function (result) {
        observer.next(result);
        observer.complete();
      })["catch"](observer.error.bind(observer));
    });
  });
}

function createErrorLink(graphQLError) {
  return new ApolloLink(function () {
    return new Observable(function (observer) {
      delay(100).then(function () {
        observer.next({
          errors: createGraphQLErrorMessage(graphQLError)
        });
        observer.complete();
      })["catch"](observer.error.bind(observer));
    });
  });
}
function createLoadingLink() {
  return new ApolloLink(function () {
    return new Observable(function () {});
  });
}
function createApolloClient(_ref) {
  var mocks = _ref.mocks,
      _ref$cacheOptions = _ref.cacheOptions,
      cacheOptions = _ref$cacheOptions === void 0 ? {} : _ref$cacheOptions,
      _ref$clientOptions = _ref.clientOptions,
      clientOptions = _ref$clientOptions === void 0 ? {} : _ref$clientOptions,
      _ref$links = _ref.links,
      links = _ref$links === void 0 ? function () {
    return [];
  } : _ref$links,
      _ref$addTypename = _ref.addTypename,
      addTypename = _ref$addTypename === void 0 ? false : _ref$addTypename;
  var mockLink;

  if (!Array.isArray(mocks)) {
    var apolloLinkOptions = {};
    var resolvers = mocks.resolvers,
        introspectionResult = mocks.introspectionResult,
        typeResolvers = mocks.typeResolvers,
        rootValue = mocks.rootValue,
        context = mocks.context,
        _delay = mocks.delay;
    var schema = buildClientSchema(introspectionResult);
    var mockOptions = {
      schema: schema
    };
    mockOptions = _extends({}, mockOptions, {
      mocks: resolvers
    });
    addMockFunctionsToSchema(mockOptions);

    if (!!typeResolvers) {
      addResolveFunctionsToSchema({
        schema: schema,
        resolvers: typeResolvers
      });
    }

    if (_delay) {
      apolloLinkOptions.delay = _delay;
    }

    mockLink = createMockLink(schema, rootValue, context, apolloLinkOptions);
  } else {
    mockLink = new MockLink(mocks, addTypename);
  }

  var cache = new InMemoryCache(_extends({}, cacheOptions, {
    addTypename: addTypename
  }));
  return new ApolloClient(_extends({
    cache: cache,
    link: ApolloLink.from([].concat(links(cache), [mockLink]))
  }, clientOptions));
}
function createGraphQLErrorMessage(graphQLError) {
  if (graphQLError) {
    return typeof graphQLError === 'string' ? [new GraphQLError(graphQLError)] : graphQLError;
  }

  return [new GraphQLError('Unspecified error from ErrorProvider.')];
}
function createMocks(query, _ref2) {
  var variables = _ref2.variables,
      data = _ref2.data,
      newData = _ref2.newData,
      graphqlErrors = _ref2.graphqlErrors,
      error = _ref2.error,
      delay = _ref2.delay;
  return [_extends({
    request: {
      query: query,
      variables: variables
    }
  }, !error ? {
    result: _extends({
      data: data
    }, graphqlErrors ? {
      errors: createGraphQLErrorMessage(graphqlErrors)
    } : {})
  } : {}, {
    error: error,
    newData: newData,
    delay: delay
  })];
}

var ApolloMockedProvider = function ApolloMockedProvider(_ref) {
  var children = _ref.children,
      _ref$Provider = _ref.Provider,
      Provider = _ref$Provider === void 0 ? ApolloProvider : _ref$Provider,
      rest = _objectWithoutPropertiesLoose(_ref, ["children", "Provider"]);

  var client = createApolloClient(rest);
  return createElement(Provider, {
    client: client
  }, children);
};

var ApolloLoadingProvider = function ApolloLoadingProvider(_ref) {
  var _ref$Provider = _ref.Provider,
      Provider = _ref$Provider === void 0 ? ApolloProvider : _ref$Provider,
      children = _ref.children;
  var client = new ApolloClient({
    link: createLoadingLink(),
    cache: new InMemoryCache()
  });
  return createElement(Provider, {
    client: client
  }, children);
};

var ApolloErrorProvider = function ApolloErrorProvider(_ref) {
  var children = _ref.children,
      graphQLError = _ref.graphQLError,
      _ref$Provider = _ref.Provider,
      Provider = _ref$Provider === void 0 ? ApolloProvider : _ref$Provider;
  var client = new ApolloClient({
    link: createErrorLink(graphQLError),
    cache: new InMemoryCache()
  });
  return createElement(Provider, {
    client: client
  }, children);
};

export { ApolloErrorProvider, ApolloLoadingProvider, ApolloMockedProvider, createApolloClient, createErrorLink, createGraphQLErrorMessage, createLoadingLink, createMocks };
//# sourceMappingURL=apollo-mocked.esm.js.map
